import fs from 'node:fs/promises';
import path from 'node:path';
import { getPlaygroundUrl } from '../shared/playgroundUrl.js';
import { startFixtureWatcher } from '../userModules/fixtureWatcher.js';
import { generateUserImports } from '../userModules/generateUserImports.js';
import { moduleExists } from '../utils/fs.js';
export const exposeImportsServerPlugin = {
    name: 'exposeImports',
    async devServer({ cosmosConfig, platformType }) {
        if (!shouldExposeImports(platformType, cosmosConfig))
            return;
        await generateImportsFile(cosmosConfig);
        const watcher = await startFixtureWatcher(cosmosConfig, 'all', () => {
            generateImportsFile(cosmosConfig);
        });
        return () => {
            watcher.close();
        };
    },
};
function shouldExposeImports(platformType, cosmosConfig) {
    return platformType === 'native' || Boolean(cosmosConfig.exposeImports);
}
async function generateImportsFile(cosmosConfig) {
    const { exposeImports } = cosmosConfig;
    const filePath = typeof exposeImports === 'string'
        ? exposeImports
        : getDefaultFilePath(cosmosConfig.rootDir);
    const typeScript = /\.tsx?$/.test(filePath);
    const rendererConfig = {
        playgroundUrl: getPlaygroundUrl(cosmosConfig),
    };
    const fileSource = generateUserImports({
        cosmosConfig,
        rendererConfig,
        relativeToDir: path.dirname(filePath),
        typeScript,
    });
    await fs.writeFile(filePath, fileSource, 'utf8');
    const relModulesPath = path.relative(process.cwd(), filePath);
    console.log(`[Cosmos] Generated ${relModulesPath}`);
}
function getDefaultFilePath(rootDir) {
    const ext = moduleExists('typescript') ? 'ts' : 'js';
    return path.resolve(rootDir, `cosmos.imports.${ext}`);
}
